function init(data)
{

    let localRatingsPage = new LocalRatingsPage(data && data.dialog);

    // Function to be called after a choice dialog is closed
    const onRebuildConfirmation = confirm => {
        if (!confirm)
            return;

        localRatingsPage.rebuildReplays();

        if (Object.keys(localRatingsPage.ratingsDatabase).length > 0 && Object.keys(localRatingsPage.historyDatabase).length > 0)
            return;

        Engine.PushGuiPage("page_localratings_choicedialog.xml", {
            "title": "No replay found",
            "text": [
                "No replay was found.",
                "The player list will show up after your first game played.",
                "If you have already played one or more games, check your filters from the \"Options\" menu."
            ],
            "confirm": "Close"
        }, data => {});
    };

    // Check if replay database is populated. If not, run the first run dialog.
    const replayDB = new LocalRatingsReplayDB();
    if (replayDB.isEmpty())
    {
        Engine.PushGuiPage("page_localratings_choicedialog.xml", {
            "title": "Welcome",
            "text": [
                "Here you will find the list of players, their rating and their statistics.",
                "The list is currently empty: click on \"Build list now\" to generate it (this may take a while)."
            ],
            "cancel": "Build list later",
            "confirm": "Build list now"
        }, data => onRebuildConfirmation(data.confirm));
        return;
    }

    // Check if database version is the current one. If not, run the update required dialog.
    const cache = new LocalRatingsCache();
    if (cache.isUpdateRequired())
    {
        Engine.PushGuiPage("page_localratings_choicedialog.xml", {
            "title": "Update required",
            "text": [
                "A new version of the LocalRatings mod has been detected.",
                "The new version requires a rebuild of the player list: click on \"Rebuild list now\" to update (this may take a while)."
            ],
            "cancel": "Rebuild list later",
            "confirm": "Rebuild list now"
        }, data => onRebuildConfirmation(data.confirm));
        return;
    }

    // All tests passed. Populate table
    localRatingsPage.updatePlayerList();

}
